'' =================================================================================================
''
''   File....... jm_countdown_demo.spin
''   Purpose....
''   Author..... Jon "JonnyMac" McPhalen
''               Copyright (C) 2015 Jon McPhalen
''               -- see below for terms of use
''   E-mail..... jon@jonmcphalen.com
''   Started....
''   Updated.... 27 JUL 2015
''
'' =================================================================================================


con { timing }

  _clkmode = xtal1 + pll16x
  _xinfreq = 5_000_000                                           ' use 5MHz crystal

  CLK_FREQ = (_clkmode >> 6) * _xinfreq                          ' system freq as a constant
  MS_001   = CLK_FREQ / 1_000                                    ' ticks in 1ms
  US_001   = CLK_FREQ / 1_000_000                                ' ticks in 1us


con { io pins }

  RX1 = 31                                                       ' programming / terminal
  TX1 = 30

  SDA = 29                                                       ' eeprom / i2c
  SCL = 28

  LED = 26


con

  DEFAULT_TIME = 3 * 60                                          ' three minutes as seconds
  

obj

' main                                                           ' * master Spin cog
  io   : "jm_io_basic"                                           '   essential io control
  time : "jm_time"                                               '   timing and delays
  term : "jm_fullduplexserial"                                   ' * serial io for terminal

' * Uses cog when loaded


var


pub main | last, now, k

  setup

  reset_timer(DEFAULT_TIME)
  last := 0                                                      ' force initial display

  repeat
    now := ||time.seconds                                        ' get time as positive
    if (now <> last)                                             ' if changed
      show_time(now)                                             ' update display
      last := now                                                ' mark updated

      
    ' active clock blinker
    ' -- blinks at 1Hz (10% duty) when clock is running

    k := ||time.millis // 1000                                   ' remove whole seconds
    if ((time.on_hold == false) and (k => 900))                  ' blink on for 1/10s
      io.high(LED)
    else
      io.low(LED)


    ' check user input

    k := term.rxcheck 
    if (k == " ")                                                ' space to hold or release
      ifnot (time.on_hold)
        time.hold
      else
        time.release
      show_time(now)

    elseif ((k == "r") or (k == "R"))                            ' r/R for reset & hold
      reset_timer(DEFAULT_TIME)
      last := 0


pub reset_timer(s)

  time.hold                    
  time.set_secs(-s)                                              ' make negative for count downn


pub show_time(s) | m

  m := s / 60                                                    ' extract minutes
  s //= 60                                                       ' extract seconds

  term.tx(term#HOME)                                             ' display time as mm:ss
  term.rjdec(m, 2, "0")
  term.tx(":")
  term.rjdec(s, 2, "0")

  if (time.on_hold)                                              ' if on hold
    term.tx(" ")                                                 '  mark display
    term.tx("*")
  else   
    term.tx(term#CLREOL)


pub setup

'' Setup IO and objects for application

  io.start($0000, $0000)                                         ' clear all pins

  time.start                                                     ' setup timing/delays for clock speed

  term.start(RX1, TX1, %0000, 115_200)                           ' start serial for terminal


dat { license }

{{

  Copyright (C) 2015 Jon McPhalen

  Terms of Use: MIT License

  Permission is hereby granted, free of charge, to any person obtaining a copy of this
  software and associated documentation files (the "Software"), to deal in the Software
  without restriction, including without limitation the rights to use, copy, modify,
  merge, publish, distribute, sublicense, and/or sell copies of the Software, and to
  permit persons to whom the Software is furnished to do so, subject to the following
  conditions:

  The above copyright notice and this permission notice shall be included in all copies
  or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
  INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
  PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
  HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
  CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE
  OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

}}